#include <stdio.h>
#include <stdlib.h>

/* defines section */
#define REQ_COMMAND_LINE_PRAM  2
#define USAGE                  "Usage example bl2_spliter file.bin \n"
#define FILE_NAME              1
#define PLAIN                  "bl2_plain.bin"
#define ENCRYPTED              "bl2_encrypted.bin"
#define KEY                    "bl2_key.bin"
#define DECRYTED_OFFSET        0xAAC
#define ENCRYTED_OFFSET        0xAA4
#define MAX_SEARCH             10
#define DATA_FOUND             1
#define DECRYTED_BYTE_0        0x48
#define DECRYTED_BYTE_1        0x00
#define DECRYTED_BYTE_2        0x00
#define DECRYTED_BYTE_3        0x99

/* prototypes section */
/* None */

int main(int argc, char *argv[])
{
    FILE *file_ptr = NULL;
    unsigned char *c_file_ptr = NULL;
    unsigned char *plain_bl2_ptr = NULL;
    unsigned char *encrypted_bl2_ptr = NULL;
    unsigned char *key_stream_bl2_ptr = NULL;
    long file_size;
    long i;
    size_t result;
    unsigned char found_decryted_data = 0;
    unsigned char data_offset = 0;

    if(argc < REQ_COMMAND_LINE_PRAM)
    {
        printf("Incorrect number of args \n");
        printf(USAGE);
        system("PAUSE");
        return 1;
    }

    file_ptr = fopen(argv[FILE_NAME],"rb");
  
    if(file_ptr == NULL)
    {
        printf("cant open file %s \n",argv[FILE_NAME]);
        system("PAUSE");
        return 1;
    }

    fseek(file_ptr,0, SEEK_END);
    file_size = ftell(file_ptr);
    rewind(file_ptr);

    /* get memory to copy file too */
    c_file_ptr = (unsigned char *) malloc((sizeof(unsigned char) * file_size));

    if(c_file_ptr == NULL)
    {
        fclose(file_ptr);
        printf("cant malloc memory to hold file ! \n");
        system("PAUSE");
        return 1;
    }

    /* copy file into memory */
    result = fread(c_file_ptr, 1, file_size, file_ptr);

    if(result != file_size)
    {
        fclose(file_ptr);
        printf("file read error %x != %x\n",result,file_size);
        system("PAUSE");
        return 1;
    }

    fclose(file_ptr);

    /* get memory to hold plain bl2 */
    plain_bl2_ptr = (unsigned char *) malloc((sizeof(unsigned char) * (file_size / 2)));

    if(plain_bl2_ptr == NULL)
    {
        free(c_file_ptr);
        printf("cant malloc memory to hold plain bl2 ! \n");
        system("PAUSE");
        return 1;
    }

    /* get memory to hold encrypted bl2 */
    encrypted_bl2_ptr = (unsigned char *) malloc((sizeof(unsigned char) * (file_size / 2)));

    if(encrypted_bl2_ptr == NULL)
    {
        free(c_file_ptr);
        free(plain_bl2_ptr);
        printf("cant malloc memory to hold encrypted bl2 ! \n");
        system("PAUSE");
        return 1;
    }

    /* split file into two halfs encrypted and plain */
    for(i = 0; i < (file_size / 2) ; i++)
    {
        plain_bl2_ptr[i] = c_file_ptr[i * 2];
    }

    /* start getting encrypted data from an odd address*/
    c_file_ptr++;

    for(i = 0; i < (file_size / 2) ; i++)
    {
        encrypted_bl2_ptr[i] = c_file_ptr[i * 2];
    }

    /* write plain bl2 to file */
    file_ptr = fopen(PLAIN,"wb");
    fwrite(plain_bl2_ptr , 1 , (file_size / 2) , file_ptr );
    fclose (file_ptr);

    /* write  encrypted bl2 to file */
    file_ptr = fopen(ENCRYPTED,"wb");
    fwrite(encrypted_bl2_ptr , 1 , (file_size / 2) , file_ptr );
    fclose (file_ptr);

    /* start of decrypted should start at offset 0xacc however       */
    /* some times because of noise (producing extra bytes )          */
    /* it starts a little later so search for decrypted data offset  */
    for(i = DECRYTED_OFFSET; i < (DECRYTED_OFFSET + MAX_SEARCH); i++)
    {
        if(plain_bl2_ptr[i] == DECRYTED_BYTE_0)
        {
            if(plain_bl2_ptr[i + 1] == DECRYTED_BYTE_1)
            {
                if(plain_bl2_ptr[i + 2] == DECRYTED_BYTE_2)
                {
                    if(plain_bl2_ptr[i + 3] == DECRYTED_BYTE_3)
                    {
                        data_offset = i - DECRYTED_OFFSET;
                        found_decryted_data = DATA_FOUND;
                        break;    /* found start off decryted data */
                    }
                }
             }
        }
    }

    /* if we found that right data at the start of the decryted section */
    /* assume dump is valid and produce a key stream (xor data) from it */
    if(found_decryted_data == DATA_FOUND)
    {   
        /* get memory to hold key stream */
        key_stream_bl2_ptr = (unsigned char *) malloc((sizeof(unsigned char) * (file_size / 2)));

        if(key_stream_bl2_ptr == NULL)
        {
            free(c_file_ptr);
            free(plain_bl2_ptr);
            free(encrypted_bl2_ptr);
            printf("cant malloc memory to hold key stream ! \n");
            system("PAUSE");
            return 1;
        }

        /* move pointer to start bl2 encrytion /decrytion */
        encrypted_bl2_ptr += ENCRYTED_OFFSET + data_offset;
        plain_bl2_ptr += DECRYTED_OFFSET + data_offset;

        for(i = 0 ; i < ((file_size / 2) - (DECRYTED_OFFSET + data_offset)) ; i++)
        {
             key_stream_bl2_ptr[i] = *encrypted_bl2_ptr ^ *plain_bl2_ptr;
             encrypted_bl2_ptr++;
             plain_bl2_ptr++;
        }

        /* write bl2 key to file */
        file_ptr = fopen(KEY,"wb");
        fwrite(key_stream_bl2_ptr , 1 , (file_size / 2) , file_ptr);
        fclose (file_ptr);

        free(key_stream_bl2_ptr);
    }
    else
    {
        printf("Cant find start of bl2 decrypted data ! \n");
    }
   
    /* clean up before exit */
    free(c_file_ptr);
    free(plain_bl2_ptr);
    free(encrypted_bl2_ptr);
}
